#!/bin/bash
# chkconfig: 2345 80 20
# description: script to apply cpu microcode
# portability changes welcome!

# vars:
#
# START			distribution specific way of kicking programs
# END			distribution specific way of checking return status
# PROGRAM		the executable to run
# ARGUMENTS		the argument we're going to call PROGRAM with
# VERBOSE		put yes/no (case sensitive) to verbose or to be quiet

# device name in LANANA
DEVICE=/dev/cpu/microcode
# device name in devfsd
DEVICE2=/dev/misc/microcode
# device name in some old versions of udev
DEVICE3=/dev/microcode
ARGUMENTS=-Qu
RETVAL=0

# Sort out the executable to run.
if [ -x /usr/sbin/microcode_ctl ]; then
	PROGRAM=/usr/sbin/microcode_ctl
elif [ -x /usr/local/sbin/microcode_ctl ]; then
	PROGRAM=/usr/local/sbin/microcode_ctl
else
# Package status: "removed" but not "purge": don't print anything, to
# not confuse the user
#	echo "$0: Can't find location of microcode_ctl binary, aborting."
	exit 5
fi

# Sort out sourcing in the distribution specific library functions
# and the command to run them.
# Debian: we make no check (for ibrid distributions)
. /etc/default/rcS
if [ "$VERBOSE" != "no" ] ; then
	END=debian_end
fi

# Lets just be sure we have a device file...
# ... with workaround because different name policy in lanana and devfsd
if [ -x /sbin/modprobe ] ; then
    /sbin/modprobe -q -k -s microcode > /dev/null 2> /dev/null || true
fi
DEV="xxx"
if [ -e "$DEVICE" ]; then 
	DEV="$DEVICE"
elif [ -e "$DEVICE2" ]; then
	DEV="$DEVICE2"
elif [ -e "$DEVICE3" ]; then
	DEV="$DEVICE3"
else
## udev is slow creating devices, so in this case we wait
## some extra time (max 70 times 0.1s = 7s).
    for i in `seq 70`; do
	if [ -e "$DEVICE" ]; then 
	    DEV="$DEVICE"
	    break
	fi
	echo -n "."
	sleep 0.1
    done
    if [ "$DEV" = "xxx" ]; then
	echo "$0: microcode device $DEVICE doesn't exist?"
	exit 0
    fi
fi

# hehehe, I really hate this :)
copyreturncode ()
{
	RETVAL=$?
	return $RETVAL
}


debian_end ()
{
	if [ $RETVAL -eq 0 ];
	then
		echo "done."
	else
		echo "failed."
	fi
}

# Generic function "a la Red Hat"
MOVE_TO_COL="echo -en \\033[60G"
SETCOLOR_SUCCESS="echo -en \\033[1;32m"
SETCOLOR_FAILURE="echo -en \\033[1;31m"
SETCOLOR_NORMAL="echo -en \\033[0;39m"

echo_success() {
	$MOVE_TO_COL
	echo -n "[  "
	$SETCOLOR_SUCCESS
	echo -n "OK"
	$SETCOLOR_NORMAL
	echo -n "  ]"
	echo -ne "\r"
	echo ""
}

echo_failure() {
	$MOVE_TO_COL
	echo -n "["
	$SETCOLOR_FAILURE
	echo -n "FAILED"
	$SETCOLOR_NORMAL
	echo -n "]"
	echo -ne "\r"
	echo ""
}


# echo OK in green if is success, FAILED in red is failed
slack_end ()
{
	if [ $RETVAL -eq 0 ];
	then
		echo
		echo -n "$0($$)"
		echo_success
	else
		echo
		echo -n "$0($$)"
		echo_failure
	fi

	if [ $RETVAL -eq 3 ]; then
		echo "Cannot find source file \"/etc/microcode.dat\""
	fi
}

# perform the update
doit ()
{
	if [ "$VERBOSE" != "no" ] ; then
		echo -n "Applying Intel IA32 Microcode update... "
	fi

	$START $PROGRAM $ARGUMENTS -d $DEV
	copyreturncode
	if [ "$VERBOSE" != "no" ] ; then
		$END
	fi

	# trap the most common case, errno 19 = no device
	if [ $RETVAL -eq 19 ];
	then
		grep MICROCODE /usr/include/linux/autoconf.h | grep -q ^#define
		if [ $? -ne 0 ];
		then
			echo "$0: This kernel doesn't appear to have microcode device support?"
		fi
	fi
	echo
	# *try* to unload the microcode module
	/sbin/modprobe -r microcode > /dev/null 2> /dev/null
}


case "$1" in
  start|""|reload|force-reload|restart)
	doit
	exit 0
	;;
  stop)
	;;
  status)
	# I wonder when the driver will let us do this...
	# maybe I'll write it ;)
	echo "$0: reading microcode status is not yet supported"
	;;
  *)
	echo "$0 usage: microcode {start|restart}"
	exit 2
esac
